@@ -18,6 +18,10 @@ class DueCreditEntry(object):
     def get_key(self):
         return self._key
 
+    @property
+    def key(self):
+        return self.get_key()
+
     @property
     def rawentry(self):
         if PY2:
